package com.qmth.distributed.print.business.templete.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.qmth.distributed.print.business.entity.ExamPrintPlan;
import com.qmth.distributed.print.business.entity.TSyncExamStudentScore;
import com.qmth.distributed.print.business.service.ExamPrintPlanService;
import com.qmth.distributed.print.business.service.TSyncExamStudentScoreService;
import com.qmth.distributed.print.business.templete.service.PushLogicService;
import com.qmth.teachcloud.common.base.BaseEntity;
import com.qmth.teachcloud.common.bean.params.UserPushParam;
import com.qmth.teachcloud.common.contant.SystemConstant;
import com.qmth.teachcloud.common.entity.BasicClazz;
import com.qmth.teachcloud.common.entity.PushUserTrack;
import com.qmth.teachcloud.common.entity.SysOrg;
import com.qmth.teachcloud.common.entity.SysUser;
import com.qmth.teachcloud.common.enums.userPush.SyncStatusEnum;
import com.qmth.teachcloud.common.service.*;
import com.qmth.teachcloud.common.sync.StmmsUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @Description: 同步推送处理逻辑接口实现类
 * @Author: CaoZixuan
 * @Date: 2021-11-01
 */
@Service
public class PushLogicServiceImpl implements PushLogicService {
    private final static Logger log = LoggerFactory.getLogger(PushLogicServiceImpl.class);

    @Resource
    private SysUserService sysUserService;

    @Resource
    private SysUserRoleService sysUserRoleService;

    @Resource
    StmmsUtils stmmsUtils;

    @Resource
    private PushUserTrackService pushUserTrackService;

    @Resource
    SysOrgService sysOrgService;

    @Resource
    BasicClazzService basicClazzService;

    @Resource
    ExamPrintPlanService examPrintPlanService;

    @Resource
    TSyncExamStudentScoreService tSyncExamStudentScoreService;

    @Override
    public Map<String, Object> executeUserPushLogic(Map<String, Object> map) throws IllegalAccessException {
        int correct = 0;
        int count = 0;
        SysUser sysUser = (SysUser) map.get(SystemConstant.SYS_USER);
        Long schoolId = sysUser.getSchoolId();
        // 需要同步的用户
        List<SysUser> sysUserList = sysUserService.list(new QueryWrapper<SysUser>().lambda()
                .eq(SysUser::getSchoolId, schoolId)
                .eq(SysUser::getSyncStatus, SyncStatusEnum.NEED_PUSH_AGAIN));
        count = sysUserList.size();
        for (SysUser user : sysUserList) {
            Set<Long> roleIdSet = sysUserRoleService.listRoleByUserId(user.getId()).stream().map(BaseEntity::getId).collect(Collectors.toSet());
            List<PushUserTrack> pushUserTrackList = pushUserTrackService.createPushTrackUser(null, roleIdSet, user.getId(), user.getEnable(), sysUser.getId());
            List<UserPushParam> userPushParamList = pushUserTrackService.analyzeUserPushSpecialPrivilege(pushUserTrackList);

            boolean syncResult = sysUserService.userPushService(userPushParamList, sysUser.getId());
            if (syncResult) {
                correct++;
            }
            UpdateWrapper<SysUser> updateWrapper = new UpdateWrapper<>();
            updateWrapper.lambda().set(SysUser::getSyncStatus, null).eq(SysUser::getId, user.getId());
            sysUserService.update(updateWrapper);
        }
        map.put("correct", correct);
        map.put("count", count);
        return map;
    }

    /**
     * 成绩查询推送异步任务处理方法
     *
     * @param map 任务源
     * @return
     * @throws IllegalAccessException
     */
    @Override
    @Transactional
    public Map<String, Object> executeScorePushLogic(Map<String, Object> map) throws Exception {
        SysUser sysUser = (SysUser) map.get(SystemConstant.USER);
        Long schoolId = sysUser.getSchoolId();
        Integer examId = (Integer) map.get("examId");
        Long examIdLong = Long.parseLong(String.valueOf(examId));
        List<TSyncExamStudentScore> tSyncExamStudentScoreList = null;
        List<String> errorTSyncExamStudentScoreList = null;
        try {
            int totalCount = stmmsUtils.getStudentCount(schoolId, examId, null, null, null, null, null, null, null, true);
            log.info("云阅卷：考试成绩考生数量查询接口调用，返回数量：{}", totalCount);
            if (totalCount > 0) {
                int pageSize = stmmsUtils.getDefaultPageSize();
                int mod = totalCount % pageSize;
                int pageNos = mod == 0 ? totalCount / pageSize : totalCount / pageSize + 1;
                Map<String, SysOrg> orgMap = new HashMap<>();
                Map<String, BasicClazz> clazzMap = new HashMap<>();
                Long semesterId = examPrintPlanService.getOne(new QueryWrapper<ExamPrintPlan>().lambda().eq(ExamPrintPlan::getThirdRelateId, examIdLong)).getId();
                tSyncExamStudentScoreList = new ArrayList<>();
                errorTSyncExamStudentScoreList = new ArrayList<>();
                for (int i = 1; i <= pageNos; i++) {
                    List<Map> students = stmmsUtils.getStudentScore(schoolId, examId, null, null, null, null, null, null, null, true, i, pageSize);
                    for (Map student : students) {
                        try {
                            //查学院id
                            Long orgId = null;
                            if (!orgMap.containsKey(student.get("college"))) {
                                SysOrg sysOrg = sysOrgService.getOne(new QueryWrapper<SysOrg>().lambda().eq(SysOrg::getSchoolId, schoolId).eq(SysOrg::getName, student.get("college")));
                                if (Objects.nonNull(sysOrg)) {
                                    orgId = sysOrg.getId();
                                    orgMap.put(sysOrg.getName(), sysOrg);
                                }
                            } else {
                                orgId = orgMap.get(student.get("college")).getId();
                            }

                            //查班级和专业id
                            Long clazzId = null, majorId = null;
                            if (!clazzMap.containsKey(student.get("className"))) {
                                BasicClazz basicClazz = basicClazzService.getOne(new QueryWrapper<BasicClazz>().lambda().eq(BasicClazz::getSchoolId, schoolId).eq(BasicClazz::getClazzName, student.get("className")));
                                if (Objects.nonNull(basicClazz)) {
                                    clazzId = basicClazz.getId();
                                    majorId = basicClazz.getMajorId();
                                    clazzMap.put(basicClazz.getClazzName(), basicClazz);
                                }
                            } else {
                                clazzId = clazzMap.get(student.get("className")).getId();
                                majorId = clazzMap.get(student.get("className")).getMajorId();
                            }
                            student.put(SystemConstant.SCHOOL_ID, schoolId);
                            student.put("semesterId", semesterId);
                            student.put("orgId", orgId);
                            student.put("clazzId", clazzId);
                            student.put("majorId", majorId);
                            student.put("userId", sysUser.getId());
                            tSyncExamStudentScoreList.add(new TSyncExamStudentScore(student));
                        } catch (Exception e) {
                            log.info("同步学生成绩信息失败:{}", e.getMessage());
                            e.printStackTrace();
                            errorTSyncExamStudentScoreList.add(student.get("name") + "," + student.get("examNumber") + "\r\n");
                        }
                    }
                }
                if (tSyncExamStudentScoreList.size() > 0) {
                    tSyncExamStudentScoreService.remove(new QueryWrapper<TSyncExamStudentScore>().lambda().eq(TSyncExamStudentScore::getSchoolId, schoolId).eq(TSyncExamStudentScore::getExamId, examIdLong));
                    tSyncExamStudentScoreService.saveOrUpdateBatch(tSyncExamStudentScoreList);
                }
            }
            map.computeIfAbsent("count", v -> totalCount);
            List<TSyncExamStudentScore> finalTSyncExamStudentScoreList = tSyncExamStudentScoreList;
            map.computeIfAbsent("correct", v -> finalTSyncExamStudentScoreList.size());
            if (Objects.nonNull(errorTSyncExamStudentScoreList) && errorTSyncExamStudentScoreList.size() > 0) {
                List<String> finalErrorTSyncExamStudentScoreList = errorTSyncExamStudentScoreList;
                map.computeIfAbsent("error", v -> "未同步成功数据" + finalErrorTSyncExamStudentScoreList.size() + "条:\r\n" + finalErrorTSyncExamStudentScoreList.toString());
            }
        } catch (Exception e) {
            log.info("云阅卷：接口调用失败：{}", e.getMessage());
            throw new RuntimeException("云阅卷：接口调用失败：" + e.getMessage());
        }
        return map;
    }
}